@@ -1755,3 +1755,121 @@ SEXP mkr2X(NumericMatrix Y, NumericMatrix K1, NumericMatrix K2){
                       Named("GC1")=GC1, Named("GC2")=GC2,
                       Named("Ve")=ve, Named("h2")=h2);}
 
+// [[Rcpp::export]]
+SEXP lasso(NumericVector y, NumericMatrix gen){
+  int maxit = 300;
+  double tol = 10e-8;
+  // Functions starts here
+  int p = gen.ncol();
+  int n = gen.nrow();
+  // Beta, mu and epsilon
+  double eM, mu = mean(y);
+  NumericVector b(p), e = y-mu;
+  // Marker variance
+  NumericVector xx(p);
+  for(int i=0; i<p; i++){xx[i] = sum(gen(_,i)*gen(_,i));}
+  double Lmb=mean(xx)/p;
+  // Convergence control
+  NumericVector bc(p), yx(p);
+  int numit = 0;
+  double cnv = 1;
+  // Loop
+  while(numit<maxit){
+    // Regression coefficients loop
+    bc = b+0;
+    for(int j=0; j<p; j++){
+      e = e+gen(_,j)*b[j];
+      yx[j] = sum(e*gen(_,j));
+      if(yx[j]>0){
+        b[j] = (yx[j]-Lmb)/xx[j];
+        if(b[j]<0){b[j]=0;}
+      }else{
+        b[j] = (yx[j]+Lmb)/xx[j];
+        if(b[j]>0){b[j]=0;}
+      }
+      e = e-gen(_,j)*b[j];}
+    // Update regularization and intercept
+    Lmb = 2*mean(abs(yx)-abs(b*xx));
+    Lmb = 2*sqrt(Lmb); // this line was added by ad hoc, without derivations
+    eM = mean(e);
+    mu = mu+eM;
+    e = e-eM;
+    // Convergence
+    ++numit;
+    cnv = sum(abs(bc-b));
+    if( cnv<tol ){break;}}
+  // Fitting the model
+  NumericVector fit = y-e;
+  double h2 = 1-(sum(e*y)/(n-1))/var(y);
+  // Output
+  return List::create(Named("mu")=mu, Named("b")=b,
+                      Named("h2")=h2, Named("hat")=fit,
+                      Named("Lmb")=Lmb);}
+
+// [[Rcpp::export]]
+SEXP emBCpi(NumericVector y, NumericMatrix gen, double df = 10, double R2 = 0.5, double Pi = 0.75){
+  int it = 200;
+  int p = gen.ncol();
+  int n = gen.nrow();
+  NumericVector d(p);
+  NumericVector b(p);
+  double vy = var(y);
+  if(Pi>0.5){Pi = 1-Pi;} 
+  NumericVector xx(p);
+  NumericVector vx(p);
+  for(int i=0; i<p; i++){
+    xx[i] = sum(gen(_,i)*gen(_,i));
+    vx[i] = var(gen(_,i));
+  }
+  double PriorPi = Pi*1.0;
+  double MSx = sum(vx)*Pi*(1-Pi);
+  double Sa = R2*(df+2)*vy/MSx;
+  double Se = (1-R2)*(df+2)*vy;
+  double mu = mean(y);
+  NumericVector e = y-mu;
+  NumericVector e1(n);
+  NumericVector e2(n);
+  double ve = Sa;
+  double va = Se;
+  double Lmb = ve/va;
+  double b0,b1,LR,eM,h2,C;
+  double Pi0 = (1-Pi)/Pi;
+  for(int i=0; i<it; i++){
+    C = -0.5/ve;
+    for(int j=0; j<p; j++){
+      b0 = b[j];
+      b1 = (sum(gen(_,j)*e)+xx[j]*b0)/(xx[j]+Lmb);
+      e1 = e-gen(_,j)*(b1-b0);
+      e2 = e-gen(_,j)*(0-b0);
+      LR = Pi0*exp(C*(sum(e2*e2)-sum(e1*e1)));
+      d[j] = (1/(1+LR));
+      b[j] = b1*d[j];
+      e = e - gen(_,j)*(b[j]-b0);
+    }
+    // update pi
+    Pi = ((1-mean(d))*p + PriorPi*df)/(p+df);
+    Pi0 = (1-Pi)/Pi;
+    MSx = sum(vx)*Pi*(1-Pi);
+    Sa = R2*(df+2)*vy/MSx;
+    // update var
+    eM = mean(e);
+    ve = (sum(e*e)+Se)/(n+df);
+    va = (sum(b*b)+Sa)/(p+df)/(mean(d)-Pi);
+    Lmb = ve/va;
+    // update mu
+    eM = mean(e);
+    mu = mu+eM;
+    e = e-eM;
+  }
+  h2 = 1-ve/vy;
+  NumericVector fit(n);
+  for(int k=0; k<n; k++){fit[k] = sum(gen(k,_)*b)+mu;}
+  return List::create(Named("mu") = mu,
+                      Named("b") = b,
+                      Named("d") = d,
+                      Named("pi") = Pi,
+                      Named("hat") = fit,
+                      Named("Vg") = va*MSx,
+                      Named("Va") = va,
+                      Named("Ve") = ve,
+                      Named("h2") = h2);}
